@@ -170,9 +170,6 @@ typedef struct intel_screen_private {
 	PixmapPtr back_pixmap;
 	unsigned int back_name;
 	long front_pitch, front_tiling;
-	void *shadow_buffer;
-	int shadow_stride;
-	DamagePtr shadow_damage;
 
 	dri_bufmgr *bufmgr;
 
@@ -328,10 +325,8 @@ typedef struct intel_screen_private {
 	Bool use_pageflipping;
 	Bool use_triple_buffer;
 	Bool force_fallback;
-	Bool can_blt;
 	Bool has_kernel_flush;
 	Bool needs_flush;
-	Bool use_shadow;
 
 	struct _DRI2FrameEvent *pending_flip[2];
 
@@ -413,9 +408,17 @@ intel_get_screen_private(ScrnInfoPtr scrn)
 	return (intel_screen_private *)(scrn->driverPrivate);
 }
 
+#ifndef ARRAY_SIZE
 #define ARRAY_SIZE(x) (sizeof(x) / sizeof(x[0]))
+#endif
+
+#ifndef ALIGN
 #define ALIGN(i,m)	(((i) + (m) - 1) & ~((m) - 1))
+#endif
+
+#ifndef MIN
 #define MIN(a,b)	((a) < (b) ? (a) : (b))
+#endif
 
 static inline unsigned long intel_pixmap_pitch(PixmapPtr pixmap)
 {
@@ -640,10 +643,6 @@ void intel_uxa_block_handler(intel_screen_private *intel);
 Bool intel_get_aperture_space(ScrnInfoPtr scrn, drm_intel_bo ** bo_table,
 			      int num_bos);
 
-/* intel_shadow.c */
-void intel_shadow_blt(intel_screen_private *intel);
-void intel_shadow_create(struct intel_screen_private *intel);
-
 static inline Bool intel_pixmap_is_offscreen(PixmapPtr pixmap)
 {
 	struct intel_pixmap *priv = intel_get_pixmap_private(pixmap);
